from viggocore.database import db
from viggocore.subsystem import domain


class DomainFiscal(domain.resource.Domain, db.Model):

    attributes = ['nfce_csc_id', 'nfce_csc',
                  'nfce_csc_id_homolog', 'nfce_csc_homolog']
    attributes += domain.resource.Domain.attributes

    id = db.Column(db.ForeignKey('domain.id'), primary_key=True)
    nfce_csc_id = db.Column(db.String(6), nullable=False)
    nfce_csc = db.Column(db.String(36), nullable=False)
    nfce_csc_id_homolog = db.Column(db.String(6), nullable=False)
    nfce_csc_homolog = db.Column(db.String(36), nullable=False)

    __mapper_args__ = {'polymorphic_identity': 'domain_fiscal'}

    def __init__(self, id, name, cnpj, nfce_csc_id, nfce_csc,
                 nfce_csc_id_homolog, nfce_csc_homolog,
                 parent_id=None, active=True, created_at=None,
                 created_by=None, updated_at=None, updated_by=None, tag=None):
        super().__init__(id, name, parent_id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.nfce_csc_id = nfce_csc_id
        self.nfce_csc = nfce_csc
        self.nfce_csc_id_homolog = nfce_csc_id_homolog
        self.nfce_csc_homolog = nfce_csc_homolog

    @classmethod
    def individual(cls):
        return 'domain_fiscal'



from viggocore.subsystem.file import manager
from vcfiscal.subsystem.sysadmin.arquivo_ibpt import tasks


class Create(manager.Create):

    def pre(self, session, **kwargs):
        self.sigla_uf = kwargs.get('sigla_uf', None)
        return super().pre(session=session, **kwargs)

    def post(self):
        tasks.process_arquivo_ibpt(self.sigla_uf,
                                   self.entity.filename,
                                   self.upload_folder,
                                   self.user_id)


class Manager(manager.Manager):

    ALLOWED_EXTENSIONS = ['csv']

    def __init__(self, driver):
        super().__init__(driver)
        self.create = Create(self)



from flask.globals import current_app
from viggocore.celery import celery, decide_on_run
from vcfiscal.subsystem.sysadmin.ibpt_sync.sync_manager import SyncManager


@decide_on_run
@celery.task
def process_arquivo_ibpt(sigla_uf: str,
                         filename: str,
                         upload_folder: str,
                         user_id: str) -> None:
    api = current_app.api_handler.api()
    sync_manager = SyncManager(api, sigla_uf, filename, upload_folder,
                               user_id)
    sync_manager.sincronizar_ibpt()



import uuid
import enum

from sqlalchemy import orm
from datetime import datetime
from sqlalchemy.types import Enum
from viggocore.database import db
from viggocore.common.subsystem import entity


class MessageType(enum.Enum):
    Info = 0
    Warning = 1
    Error = 2


class Status(enum.Enum):
    Pendente = 0
    Sucesso = 1
    Erro = 2


class IBPTSync(entity.Entity, db.Model):

    attributes = ['ibpt_id', 'status', 'sigla_uf', 'id']

    ibpt_id = db.Column(db.CHAR(32), db.ForeignKey("ibpt.id"), nullable=True)
    ibpt = orm.relationship("IBPT", backref=orm.backref('ibpt'))
    messages = orm.relationship(
        "IBPTSyncMessage", backref=orm.backref('ibpt_sync'),
        cascade='delete,delete-orphan,save-update')
    status = db.Column(Enum(Status), nullable=False)
    sigla_uf = db.Column(db.CHAR(2), nullable=False)

    def __init__(self, id, sigla_uf, ibpt_id=None, status=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.sigla_uf = sigla_uf
        self.status = status if status else Status.Pendente
        self.ibpt_id = ibpt_id

    def addMsg(self, msg_type, msg_body):
        message = IBPTSyncMessage(
            id=uuid.uuid4().hex, ibpt_sync_id=self.id,
            created_at=datetime.now(), type=msg_type, body=msg_body)
        self.messages.append(message)

    def contains_error(self):
        error = False
        for message in self.messages:
            if message.type == MessageType.Error:
                error = True
                break
        return error

    @classmethod
    def individual(cls):
        return 'ibpt_sync'

    @classmethod
    def embedded(cls):
        return ['messages']


class IBPTSyncMessage(entity.Entity, db.Model):

    attributes = ['ibpt_sync_id', 'type', 'body']
    attributes += entity.Entity.attributes

    ibpt_sync_id = db.Column(
        db.CHAR(32), db.ForeignKey("ibpt_sync.id"), nullable=False)
    type = db.Column(Enum(MessageType), nullable=False)
    body = db.Column(db.String(250), nullable=False)

    def __init__(self, id, ibpt_sync_id, type, body,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by)
        self.ibpt_sync_id = ibpt_sync_id
        self.type = type
        self.body = body

    @classmethod
    def individual(cls):
        return 'ibpt_sync_message'



SYSADMIN_EXCLUSIVE_POLICIES = [
    ('/arquivos_ibpt', ['POST']),
    ('/arquivos_ibpt/<id>', ['PUT', 'DELETE']),

    ('/ibpt_syncs', ['POST', 'GET']),
    ('/ibpt_syncs/<id>', ['PUT', 'GET', 'DELETE']),
]
SYSADMIN_RESOURCES = [

]
USER_RESOURCES = []

# *ibpt* sysadmin
# ibpt, ibpt_ncm, GET ALL



from viggocore.database import db
from viggocore.subsystem.file.resource import File


class ArquivoIBPT(File, db.Model):

    attributes = ['sigla_uf']
    attributes += File.attributes

    id = db.Column(db.ForeignKey('file_infosys.id'), primary_key=True)
    sigla_uf = db.Column(db.CHAR(2), nullable=False)

    __mapper_args__ = {'polymorphic_identity': 'arquivo_ibpt'}

    __tablename__ = 'arquivo_ibpt'

    def __init__(self, id, domain_id, name, sigla_uf,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, domain_id, name, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_id
        self.sigla_uf = sigla_uf

    @classmethod
    def individual(cls):
        return 'arquivo_ibpt'

    @classmethod
    def collection(cls):
        return 'arquivos_ibpt'



from sqlalchemy import UniqueConstraint

from sqlalchemy import orm
from viggocore.database import db
from viggocore.common.subsystem import entity


class NCM_CEST(entity.Entity, db.Model):

    attributes = ['ncm_id', 'cest_id']
    attributes += entity.Entity.attributes

    ncm_id = db.Column(
        db.CHAR(32), db.ForeignKey('ncm.id'), nullable=False)
    ncm = orm.relationship(
        'NCM', backref=orm.backref('ncm_cest_ncm'))
    cest_id = db.Column(
        db.CHAR(32), db.ForeignKey('cest.id'), nullable=False)
    cest = orm.relationship(
        'CEST', backref=orm.backref('ncm_cest_cest'))

    __tablename__ = 'ncm_cest'

    __table_args__ = (
        UniqueConstraint('ncm_id', 'cest_id',
                         name='ncm_cest_ncm_id_cest_id_uk'),)

    def __init__(self, id, ncm_id, cest_id,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.ncm_id = ncm_id
        self.cest_id = cest_id

    @classmethod
    def individual(cls):
        return 'ncm_cest'



from sqlalchemy import orm, UniqueConstraint

from viggocore.common.subsystem import entity
from viggocore.database import db


class IBPTNCM(entity.Entity, db.Model):

    attributes = ['ibpt_id', 'ncm_id', 'sigla_uf', 'aliq_nac_fed',
                  'aliq_imp_fed', 'aliq_est', 'aliq_mun']
    attributes += entity.Entity.attributes

    ibpt_id = db.Column(
        db.CHAR(32), db.ForeignKey("ibpt.id"), nullable=False)
    ibpt = orm.relationship("IBPT", backref=orm.backref('ibpt_ncm_ibpt'))
    ncm_id = db.Column(
        db.CHAR(32), db.ForeignKey("ncm.id"), nullable=False)
    ncm = orm.relationship("NCM", backref=orm.backref('ibpt_ncm_ncm'))
    sigla_uf = db.Column(db.CHAR(2), nullable=False)
    aliq_nac_fed = db.Column(db.Numeric(6, 2), nullable=False)
    aliq_imp_fed = db.Column(db.Numeric(6, 2), nullable=False)
    aliq_est = db.Column(db.Numeric(6, 2), nullable=False)
    aliq_mun = db.Column(db.Numeric(6, 2), nullable=False)

    __tablename__ = 'ibpt_ncm'
    __table_args__ = (UniqueConstraint(
        'ibpt_id', 'ncm_id', 'sigla_uf',
        name='ibpt_ncm_ibpt_id_ncm_id_sigla_uf_uk'),)

    def __init__(self, id, ibpt_id, ncm_id, sigla_uf, aliq_nac_fed,
                 aliq_imp_fed, aliq_est, aliq_mun, active=True, created_at=None,
                 created_by=None, updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.ibpt_id = ibpt_id
        self.ncm_id = ncm_id
        self.sigla_uf = sigla_uf
        self.aliq_nac_fed = aliq_nac_fed
        self.aliq_imp_fed = aliq_imp_fed
        self.aliq_est = aliq_est
        self.aliq_mun = aliq_mun

    @classmethod
    def individual(cls):
        return 'ibpt_ncm'



from viggocore.common import subsystem
from vcfiscal.subsystem.sysadmin.ibpt_sync import resource, manager

subsystem = subsystem.Subsystem(resource=resource.IBPTSync,
                                manager=manager.Manager)



import enum

from sqlalchemy.types import Enum

from viggocore.common.subsystem import entity
from viggocore.database import db


class REGIME_PIS_COFINS(enum.Enum):
    CUMULATIVO = 0
    NAO_CUMULATIVO = 1
    MEI = 2
    SIMPLES = 3


class RegimeTributario(entity.Entity, db.Model):
    attributes = ['domain_id', 'crt', 'descricao', 'regime_pis_cofins',
                  'aliq_pis', 'aliq_cofins', 'contrib_ipi']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)
    crt = db.Column(db.Numeric(1), nullable=False)
    descricao = db.Column(db.String(100), nullable=False)
    regime_pis_cofins = db.Column(Enum(REGIME_PIS_COFINS), nullable=False)
    aliq_pis = db.Column(db.Numeric(5, 2), nullable=True)
    aliq_cofins = db.Column(db.Numeric(5, 2), nullable=True)
    contrib_ipi = db.Column(db.Boolean, nullable=True)

    def __init__(self, id, domain_id, crt, descricao, regime_pis_cofins,
                 aliq_pis, aliq_cofins, contrib_ipi,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag=None)
        self.domain_id = domain_id
        self.crt = crt
        self.descricao = descricao
        self.regime_pis_cofins = regime_pis_cofins
        self.aliq_pis = aliq_pis
        self.aliq_cofins = aliq_cofins
        self.contrib_ipi = contrib_ipi

    @classmethod
    def individual(cls):
        return 'regimes_tributario'



import flask

from viggocore.subsystem.file import controller
from viggocore.subsystem.file.controller import get_bad_request


class Controller(controller.Controller):

    def create(self, **kwargs):
        sigla_uf = flask.request.form.get('sigla_uf', None)
        entity_name = flask.request.form.get('entity_name', None)
        if not sigla_uf and entity_name == 'ibpt':
            return get_bad_request('sigla_uf é obrigatório no sync de ibpt')

        kwargs['sigla_uf'] = sigla_uf
        kwargs['entity_name'] = entity_name

        return super().create(**kwargs)



import enum

from sqlalchemy import UniqueConstraint, Enum, orm

from viggocore.common.subsystem import entity
from viggocore.database import db


class DESTINATARIO(enum.Enum):
    CONTRIBUINTE = 0
    NAO_CONTRIBUINTE = 1
    CONTRIBUINTE_ISENTO = 2


# TODO melhorar a sequencia em codigo
class OperacaoFiscal(entity.Entity, db.Model):
    attributes = ["regime_tributario_id", "ncm_id", "cfop_id", "codigo",
                  "descricao", "destinatario", "origem", "cst_icms",
                  "csosn_icms", "cest", "mva", "cst_pis", "aliq_ipi", "fcp",
                  "aliq_icms_interna", "aliq_icms_externa", "diferimento",
                  "aliq_pis", "cst_cofins", "aliq_cofins", "texto_legal"]
    attributes += entity.Entity.attributes

    regime_tributario_id = db.Column(
        db.CHAR(32), db.ForeignKey('regime_tributario.id'), nullable=False)
    regime_tributario = orm.relationship(
        "RegimeTributario",
        backref=orm.backref('operacao_fiscal_regime_tributario'))
    ncm_id = db.Column(db.CHAR(32), db.ForeignKey('ncm.id'), nullable=False)
    ncm = orm.relationship("NCM", backref=orm.backref('operacao_fiscal_ncm'))
    cfop_id = db.Column(db.CHAR(32), db.ForeignKey('cfop.id'), nullable=False)
    cfop = orm.relationship("CFOP",
                            backref=orm.backref('operacao_fiscal_cfop'))
    codigo = db.Column(db.Numeric(6), db.Sequence('operacao_fiscal_codigo_sq'),
                       nullable=False)
    descricao = db.Column(db.String(100), nullable=False)
    destinatario = db.Column(Enum(DESTINATARIO), nullable=True)
    origem = db.Column(db.Numeric(1), nullable=False)
    cst_icms = db.Column(db.CHAR(2), nullable=True)
    csosn_icms = db.Column(db.CHAR(3), nullable=True)
    cest = db.Column(db.CHAR(7), nullable=True)
    aliq_icms_interna = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_icms_externa = db.Column(db.Numeric(5, 2), nullable=False)
    mva = db.Column(db.Numeric(5, 2), nullable=True)
    diferimento = db.Column(db.Numeric(5, 2), nullable=True)
    fcp = db.Column(db.Numeric(5, 2), nullable=True)
    aliq_ipi = db.Column(db.Numeric(5, 2), nullable=True)
    cst_pis = db.Column(db.CHAR(2), nullable=False)
    aliq_pis = db.Column(db.Numeric(6, 2), nullable=True)
    cst_cofins = db.Column(db.CHAR(2), nullable=False)
    aliq_cofins = db.Column(db.Numeric(6, 2), nullable=True)
    texto_legal = db.Column(db.String(200), nullable=True)

    __table_args__ = (
        UniqueConstraint('regime_tributario_id', 'codigo',
                         name='operacao_fiscal_reg_trib_id_codigo_uk'),
        UniqueConstraint('regime_tributario_id', 'ncm_id', 'cfop_id',
                         name='operacao_fiscal_reg_trib_id_ncm_id_cfop_id_uk'),
    )

    def __init__(self, id, regime_tributario_id, ncm_id, cfop_id, codigo,
                 descricao, origem, aliq_icms_interna, aliq_icms_externa,
                 cst_pis, cst_cofins, destinatario=None, cst_icms=None,
                 csosn_icms=None, cest=None, mva=None, diferimento=None,
                 fcp=None, aliq_ipi=None, aliq_pis=None, aliq_cofins=None,
                 texto_legal=None, active=True, created_at=None,
                 created_by=None, updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.regime_tributario_id = regime_tributario_id
        self.ncm_id = ncm_id
        self.cfop_id = cfop_id
        self.codigo = codigo
        self.descricao = descricao
        self.origem = origem
        self.aliq_icms_interna = aliq_icms_interna
        self.aliq_icms_externa = aliq_icms_externa
        self.cst_pis = cst_pis
        self.cst_cofins = cst_cofins
        self.destinatario = destinatario
        self.cst_icms = cst_icms
        self.csosn_icms = csosn_icms
        self.cest = cest
        self.mva = mva
        self.diferimento = diferimento
        self.fcp = fcp
        self.aliq_ipi = aliq_ipi
        self.aliq_pis = aliq_pis
        self.aliq_cofins = aliq_cofins
        self.texto_legal = texto_legal

    @classmethod
    def individual(cls):
        return "operacao_fiscal"

    @classmethod
    def collection(cls):
        return "operacao_fiscais"



import os
import viggocore
import vclocal

from flask_cors import CORS

from viggocore.system import System

from vcfiscal.subsystem.sysadmin import ibpt_sync, arquivo_ibpt
from vcfiscal.subsystem.parametrizacao \
    import domain_fiscal, ibpt, cfop, cest, ncm, \
    regime_tributario, operacao_fiscal, ibpt_ncm, ncm_cest
from vcfiscal.resources import SYSADMIN_EXCLUSIVE_POLICIES, \
    SYSADMIN_RESOURCES, USER_RESOURCES


system = System('vcfiscal',
                [domain_fiscal.subsystem, ibpt.subsystem, ibpt_sync.subsystem,
                 cfop.subsystem, cest.subsystem, ncm.subsystem,
                 regime_tributario.subsystem, operacao_fiscal.subsystem,
                 ibpt_ncm.subsystem, arquivo_ibpt.subsystem, ncm_cest.subsystem
                 ],
                USER_RESOURCES,
                SYSADMIN_RESOURCES,
                SYSADMIN_EXCLUSIVE_POLICIES)


class SystemFlask(viggocore.SystemFlask):

    def __init__(self):
        super().__init__(system, vclocal.system)
        #     # VCLocal
        #     uf.subsytem,

        #     # VCFiscal
        #     domain_fiscal, ibpt.subsystem, ibpt_sync.subsystem,
        #     cfop.subsystem, cest.subsystem, ncm.subsystem,
        #     regime_tributario.subsystem, operacao_fiscal.subsystem,
        #     ibpt_ncm.subsystem, arquivo_ibpt.subsystem
        # )

    def configure(self):
        origins_urls = os.environ.get('ORIGINS_URLS', '*')
        CORS(self, resources={r'/*': {'origins': origins_urls}})

        self.config['BASEDIR'] = os.path.abspath(os.path.dirname(__file__))
        self.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = True
        vcfiscal_database_uri = os.getenv('VCFISCAL_DATABASE_URI', None)
        if vcfiscal_database_uri is None:
            raise Exception('VCFISCAL_DATABASE_URI not defined in enviroment.')
        else:
            # URL os enviroment example for MySQL
            # export VCFISCAL_DATABASE_URI=
            # mysql+pymysql://root:mysql@localhost:3306/vcfiscal
            self.config['SQLALCHEMY_DATABASE_URI'] = vcfiscal_database_uri



from viggocore.common import subsystem
from vcfiscal.subsystem.sysadmin.arquivo_sync import manager, resource, \
    controller

subsystem = subsystem.Subsystem(manager=manager.Manager,
                                controller=controller.Controller,
                                resource=resource.ArquivoSync)



from flask.globals import current_app
from viggocore.celery import celery, decide_on_run
from vcfiscal.subsystem.sysadmin.arquivo_sync.sync_managers.sync_manager_ibpt \
    import SyncManagerIBPT


@decide_on_run
@celery.task
def process_arquivo_sync(filename: str,
                         upload_folder: str,
                         user_id: str,
                         entity_name: str,
                         sigla_uf: None) -> None:
    api = current_app.api_handler.api()

    if entity_name == 'ibpt':
        sync_manager_ibpt = SyncManagerIBPT(
            api, sigla_uf, filename, upload_folder, user_id)
        sync_manager_ibpt.sincronizar_ibpt()
    elif entity_name == 'cfop':
        pass
    elif entity_name == 'cest':
        pass



from viggocore.common.subsystem import manager, operation
from vcfiscal.subsystem.parametrizacao.ibpt.resource import IBPT
from vcfiscal.subsystem.sysadmin.ibpt_sync.resource import Status, IBPTSync


class VerifySync(operation.Count):

    def pre(self, session, **kwargs):
        self.chave = kwargs.get('chave', None)
        self.versao = kwargs.get('versao', None)
        self.sigla_uf = kwargs.get('sigla_uf', None)
        return super().pre()

    def do(self, session, **kwargs):
        query = session.query(IBPTSync). \
            join(IBPT, IBPTSync.ibpt_id == IBPT.id). \
            filter(IBPT.chave == self.chave). \
            filter(IBPT.versao == self.versao). \
            filter(IBPTSync.sigla_uf == self.sigla_uf). \
            filter(IBPTSync.status == Status.Sucesso). \
            distinct()
        result = query.count()

        return result


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.verify_sync = VerifySync(self)



from sqlalchemy import UniqueConstraint

from viggocore.database import db
from viggocore.common.subsystem import entity


class NCM(entity.Entity, db.Model):

    attributes = ['codigo', 'ex', 'descricao', 'tipo', 'cest_id']
    attributes += entity.Entity.attributes

    codigo = db.Column(db.CHAR(9), nullable=False)
    ex = db.Column(db.VARCHAR(2), nullable=False)
    descricao = db.Column(db.String(500), nullable=False)
    tipo = db.Column(db.Numeric(1), nullable=False)

    __tablename__ = 'ncm'

    __table_args__ = (
        UniqueConstraint('codigo', 'ex', name='ncm_codigo_ex_uk'),)

    def __init__(self, id, codigo, descricao, tipo, ex='', cest_id=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.cest_id = cest_id
        self.codigo = codigo
        self.ex = ex
        self.descricao = descricao
        self.tipo = tipo



import flask

from viggocore.subsystem.file import controller
from viggocore.subsystem.file.controller import get_bad_request


class Controller(controller.Controller):

    def create(self, **kwargs):
        sigla_uf = flask.request.form.get('sigla_uf', None)
        if not sigla_uf:
            return get_bad_request('sigla_uf é obrigatório')

        kwargs['sigla_uf'] = sigla_uf

        return super().create(**kwargs)
        # try:
        #     file = flask.request.files.get('file', None)
        #     if not file:
        #         return get_bad_request('file é obrigatório')
        #
        #     domain_id = self.get_domain_id()
        #     if not domain_id:
        #         return get_bad_request(
        #             'Não foi possível determinar o domain_id')
        #
        #     sigla_uf = flask.request.form.get('sigla_uf', None)
        #     if not sigla_uf:
        #         return get_bad_request('sigla_uf é obrigatório')
        #
        #     user_id = self.get_token(self.get_token_id()).user_id
        #     if not user_id:
        #         return get_bad_request()
        #
        #     entity = self.manager.create(file=file, domain_id=domain_id,
        #                                  sigla_uf=sigla_uf, user_id=user_id)
        # except exception.ViggoCoreException as exc:
        #     return flask.Response(response=exc.message,
        #                           status=exc.status)
        #
        # response = {self.resource_wrap: entity.to_dict()}
        #
        # return flask.Response(response=json.dumps(response, default=str),
        #                       status=201,
        #                       mimetype="application/json")



import os
import csv
import uuid
from datetime import datetime
from viggocore.common.exception import BadRequest, NotFound
from vcfiscal.subsystem.parametrizacao.ncm.resource import NCM
from vcfiscal.subsystem.parametrizacao.ibpt.resource import IBPT
from vcfiscal.subsystem.parametrizacao.ibpt_ncm.resource import IBPTNCM
from vcfiscal.subsystem.sysadmin.ibpt_sync.resource import MessageType, \
    Status, IBPTSync


class SyncManager():

    def __init__(self, api, sigla_uf: str, filename: str, upload_folder: str,
                 user_id: str):
        self.api = api
        self.sigla_uf = sigla_uf
        self.user_id = user_id
        file = self.get_file(filename, upload_folder)

        # pegar as linhas do arquivo em listas para processar
        with open(file, mode='r', encoding='ISO-8859-1') as csv_file:
            # preparando as informações
            self.get_listas(csv_file)

    def sincronizar_ibpt(self):
        # montar a classe IBPTSync
        self.ibpt_sync = IBPTSync(uuid.uuid4().hex,
                                  sigla_uf=self.sigla_uf,
                                  created_at=datetime.now(),
                                  created_by=self.user_id)

        self.ibpt_sync.addMsg(
            MessageType.Info,
            self.make_msg('Iniciando a sincronização do IBPT')
        )

        # verificar se já tem sincronização para este arquivo
        self.check_ibpt_processado()

        if not self.ibpt_sync.contains_error():
            try:
                self.api.ibpt_syncs().driver.transaction_manager.count += 1
                # get ibpt
                ibpt = self.get_ibpt()

                if ibpt is not None:
                    self.ibpt_sync.ibpt_id = ibpt.id

                    # cadastrar os ncm
                    self.register_ncms(ibpt)

                    # cadastrar os nbs
                    # self.register_nbs(ibpt)

                    # cadastrar os lc116
                    # self.register_lc116(ibpt)
                self.api.ibpt_syncs().driver.transaction_manager.commit()
            except Exception:
                self.api.ibpt_syncs().driver.transaction_manager.rollback()
                self.api.ibpt_syncs().driver.transaction_manager.count = 0

        # cadastrar o IBPTSync
        if self.ibpt_sync.contains_error():
            self.ibpt_sync.ibpt_id = None
            self.ibpt_sync.status = Status.Erro
            self.ibpt_sync.addMsg(
                MessageType.Error,
                self.make_msg('Sincronização falhou!')
            )
        else:
            self.ibpt_sync.status = Status.Sucesso
            self.ibpt_sync.addMsg(
                MessageType.Info,
                self.make_msg('Sincronização concluida!')
            )

        self.api.ibpt_syncs().create(**self.ibpt_sync.to_dict())

    def get_now(self):
        return f'[{datetime.now()}] '

    def make_msg(self, msg: str):
        return self.get_now() + msg

    def get_file(self, filename: str, upload_folder: str):
        if not filename:
            raise BadRequest()

        file = os.path.join(upload_folder, filename)
        if not os.path.exists(file):
            error = NotFound()
            error.message = 'File {} Not Found'.format(filename)
            raise error

        return file

    def get_listas(self, csv_file):
        self.first_line_file = {}
        self.ncm_lines_file = []
        self.nbs_lines_file = []
        self.lc116_lines_file = []

        ibpt_file_lines = csv.DictReader(csv_file, delimiter=';')

        line_count = 0
        for ibpt_file_line in ibpt_file_lines:
            if line_count > 0:
                # INFO: o ibpt pode ter os seguintes tipos:
                #     0 -> NCM
                #     1 -> NBS
                #     2 -> LC116
                # monta a lista de ncm
                if ibpt_file_line['tipo'] == '0':
                    self.ncm_lines_file.append(ibpt_file_line)

                # monta a lista de nbs
                if ibpt_file_line['tipo'] == '1':
                    self.nbs_lines_file.append(ibpt_file_line)

                # monta a lista de lc116
                if ibpt_file_line['tipo'] == '2':
                    self.lc116_lines_file.append(ibpt_file_line)
            else:
                line_count += 1
                # pega a primeira linha do arquivo
                self.first_line_file = ibpt_file_line

    def get_ibpt(self):
        chave = self.first_line_file['chave']
        versao = self.first_line_file['versao']
        ibpts = self.api.ibpts().list(chave=chave, versao=versao)

        self.ibpt_sync.addMsg(MessageType.Info,
                              self.make_msg('Cadastrando IBPT {}.{} {}'.format(
                                chave, versao, self.sigla_uf)))
        try:
            if not ibpts:
                fonte = self.first_line_file['fonte']
                data_ini_vigencia = datetime.strptime(
                    self.first_line_file['vigenciainicio'], '%d/%m/%Y')
                data_fim_vigencia = datetime.strptime(
                    self.first_line_file['vigenciafim'], '%d/%m/%Y')

                ibpt = IBPT(
                    id=uuid.uuid4().hex,
                    data_ini=data_ini_vigencia,
                    data_fim=data_fim_vigencia,
                    chave=chave,
                    versao=versao,
                    fonte=fonte,
                    created_at=datetime.now(),
                    created_by=self.user_id)
                self.api.ibpts().create(**ibpt.to_dict())
            else:
                ibpt = ibpts[0]
        except Exception:
            ibpt = None
            self.ibpt_sync.addMsg(MessageType.Error,
                                  self.make_msg(
                                    'Erro ao cadastrar o IBPT {}.{} {}'.format(
                                        chave, versao, self.sigla_uf)))

        return ibpt

    def check_ibpt_processado(self):
        # verifica se um mesmo arquivo com essa chave e versão já
        # foi sincronizado para essa uf
        filter_dict = {
            'chave': self.first_line_file['chave'],
            'versao': self.first_line_file['versao'],
            'sigla_uf': self.sigla_uf
        }
        # exists é 0 ou 1, 0 não existe e 1 para existe
        exists = self.api.ibpt_syncs().verify_sync(**filter_dict)

        if exists:
            msg = ('IBPT Sync já foi realizado para IBPT {}.{} {}')
            self.ibpt_sync.addMsg(
                MessageType.Error,
                self.make_msg(
                    msg.format(self.first_line_file['chave'],
                               self.first_line_file['versao'],
                               self.sigla_uf)))

    def get_ncm(self, ncm_line):
        # ex = None
        # if ibpt_ncm["ex"].strip() != '':
        #     ex = ibpt_ncm["ex"].strip()
        ex = ncm_line['ex'].strip()
        codigo = ncm_line['codigo']
        descricao = ncm_line['descricao']
        tipo = ncm_line['tipo']
        ncms = self.api.ncms().list(codigo=codigo, ex=ex)
        if not ncms:
            ncm = NCM(
                id=uuid.uuid4().hex,
                codigo=codigo,
                ex=ex,
                descricao=descricao,
                tipo=tipo,
                created_at=datetime.now(),
                created_by=self.user_id)
            self.api.ncms().create(**ncm.to_dict())
        else:
            ncm = ncms[0]

        return ncm

    def get_ibpt_ncm(self, ibpt_id, ncm_id, ncm_line):
        aliq_nac_fed = ncm_line['nacionalfederal']
        aliq_imp_fed = ncm_line['importadosfederal']
        aliq_est = ncm_line['estadual']
        aliq_mun = ncm_line['municipal']

        ibpt_ncms = self.api.ibpt_ncms().list(
            ibpt_id=ibpt_id, ncm_id=ncm_id, sigla_uf=self.sigla_uf)
        if not ibpt_ncms:
            ibpt_ncm = IBPTNCM(
                id=uuid.uuid4().hex,
                ibpt_id=ibpt_id,
                ncm_id=ncm_id,
                sigla_uf=self.sigla_uf,
                aliq_nac_fed=aliq_nac_fed,
                aliq_imp_fed=aliq_imp_fed,
                aliq_est=aliq_est,
                aliq_mun=aliq_mun,
                created_at=datetime.now(),
                created_by=self.user_id)
            self.api.ibpt_ncms().create(**ibpt_ncm.to_dict())
        else:
            ibpt_ncm = ibpt_ncms[0]

        return ibpt_ncm

    def register_ncms(self, ibpt):
        self.ibpt_sync.addMsg(
            MessageType.Info,
            self.make_msg(
                'Iniciando processamento dos ncm'))

        for ncm_line in self.ncm_lines_file:
            try:
                ncm = self.get_ncm(ncm_line)
                if ncm:
                    self.get_ibpt_ncm(ibpt.id, ncm.id, ncm_line)
            except Exception as e:
                msg = ('Error: codigo_ncm={}, \ndescricao={}')
                self.ibpt_sync.addMsg(
                    MessageType.Error,
                    self.make_msg(
                        msg.format(ncm_line['codigo'],
                                   str(e)[:150])))
                raise e

        self.ibpt_sync.addMsg(
            MessageType.Info,
            self.make_msg('Finalizado o processamento dos ncm'))



from sqlalchemy import UniqueConstraint
from viggocore.database import db
from viggocore.common.subsystem import entity


class IBPT(entity.Entity, db.Model):

    attributes = ['data_ini', 'data_fim', 'chave', 'versao', 'fonte']
    attributes += entity.Entity.attributes

    data_ini = db.Column(db.Date, nullable=False)
    data_fim = db.Column(db.Date, nullable=False)
    chave = db.Column(db.String(10), nullable=False)
    versao = db.Column(db.String(10), nullable=False)
    fonte = db.Column(db.String(100), nullable=False)

    __tablename__ = 'ibpt'
    __table_args__ = (
        UniqueConstraint('chave', 'versao', name='ibpt_chave_versao_uk'),)

    def __init__(self, id, data_ini, data_fim, chave, versao, fonte,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)

        self.data_ini = self.allDateFmtFromAllTypes(data_ini)
        self.data_fim = self.allDateFmtFromAllTypes(data_fim)
        self.chave = chave
        self.versao = versao
        self.fonte = fonte
